package com.hzya.frame.nifiapi.operation;

import com.hzya.frame.nifiapi.model.joincreateprocessconnection.CreateProcessorConnections;
import com.hzya.frame.nifiapi.model.joincreparamcontext.CreateParamContextJoin;
import com.hzya.frame.nifiapi.model.joinfindneedmodifycontroller.FindNeedModifyController;
import com.hzya.frame.nifiapi.model.joingetallcontrollerservice.ControllerServices19;
import com.hzya.frame.nifiapi.model.processgroupid.Processors;
import com.hzya.frame.nifiapi.model.resultNeedModifyController.PortFilterResult;

import java.util.List;

/**
 * Nifi基础操作封装
 *
 * @Author：liuyang
 * @Package：com.hzya.frame.nifiapi.operation
 * @Project：fw-nifi
 * @name：BaseNifiOperation
 * @Date：2025/5/21 17:50
 * @Filename：BaseNifiOperation
 */
public interface INifiBaseOperation {
    /**
     * 实例化某个应用（对应nifi流程组id）
     *
     * @param appProcessGroupId        需要复制的流程组id(应用id)，比如Oracle应用id、Mysql应用id
     * @param parentProcessGroupId     流程组id(应用id)的父流程组id
     * @param copyTargetProcessGroupId 指定创建的流程组id
     * @return 返回流程组实例化id
     */
    String instantiateApp(String appProcessGroupId, String parentProcessGroupId, String copyTargetProcessGroupId) throws Exception;

    /**
     * 拷贝业务处理器
     *
     * @param targetProcessorId        目标处理器id
     * @param parentProcessGroupId     目标处理器id的父处理器id
     * @param copyTargetProcessGroupId 需要复制到指定目标流程组的id
     */
    String copyProcessor(String targetProcessorId, String parentProcessGroupId, String copyTargetProcessGroupId) throws Exception;

    /**
     * 新增上下文参数环境，并绑定到指定的流程组
     *
     * @param createParamContextJoin 绑定上下文参数封装对象
     * @return 返回被绑定参数上下文的流程组id
     */
    String createParameterContextsBindingProcessGroup(CreateParamContextJoin createParamContextJoin) throws Exception;

    /**
     * 查找带有接口修改标识的控制器（连接池或其他的控制器）详情
     *
     * @param processGroupId 流程组id
     */
    List<ControllerServices19> getNeedModifyControllerServicesByProcessGroup(String processGroupId) throws Exception;

    /**
     * 查找带有接口修改标记的控制器服务，eg：接口修改标记:CZFJwnb9sb
     *
     * @param processGroupId 流程组id
     * @param mark           接口修改标记字符串
     */
    ControllerServices19 getNeedModifyControllerServicesByProcessGroup(String processGroupId, String mark) throws Exception;

    /**
     * 查找带有接口关系标记的输入和输出，eg：接口关系标记:f5228bc8-0360-41eb-a640-9f4df953937c
     *
     * @param processGroupId 流程组id
     * @param mark           接口修改标记字符串，如果为null，则不进行过滤
     */
    PortFilterResult getNeedModifyControllerRelationshipByProcessGroup(String processGroupId, String mark) throws Exception;

    /**
     * 查找带"接口状态清理标记"的处理器
     *
     * @param processGroupId 流程组id
     * @param mark           接口修改标记字符串，如果为null，则不进行过滤
     */
    List<Processors> getFindStateClearMarkProcessor(String processGroupId, String mark) throws Exception;

    /**
     * 查找某个流程组(应用app)内,带有"修改标记"的控制器服务，并更新控制器，从而达到实例化要求
     *
     * @param findNeedModifyController 控制器更新封装
     * @param isThrowError             true抛出异常，false不抛出异常、假设某个流程下有3个控制器服务需要修改，但入参只传了2个，在不满足业务需求且isError=true的情况下，就会抛出异常，主要是提醒业务调用代码
     * @param enabledAll               是否激活当前流程组下的所有控制器服务 true是 false否
     */
    void findNeedModifyControllerAndChangesOccur(FindNeedModifyController findNeedModifyController, boolean isThrowError, boolean enabledAll) throws Exception;

    /**
     * 批量创建某个流程下的连接关系
     *
     * @param createProcessorConnections 连接关系封装
     */
    void batchCreateProcessorConnections(CreateProcessorConnections createProcessorConnections) throws Exception;

    /**
     * 批量激活/停用某个流程组下的所有控制器服务
     *
     * @param processGroupId 流程组id
     * @param status         ENABLED 启用、DISABLED停用
     */
    void batchEnabledControllerServices(String processGroupId, String status) throws Exception;

    /**
     * 清理掉拷贝后新的流程组中的老鼠屎
     *
     * @param processGroupIdList 流程组id集合
     */
    void clearProcessGroupState(List<String> processGroupIdList) throws Exception;
}